/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific
   main-programmer: Patrick Boenzli
   co-programmer: Benjamin Grauer

*/
#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_WEAPON

#include "hbolt.h"

#include "state.h"
#include "model.h"

#include "particles/dot_emitter.h"
#include "particles/sprite_particles.h"

#include <cassert>
#include "debug.h"

#include "static_model.h"
#include "effects/billboard.h"


#include "class_id_DEPRECATED.h"
ObjectListDefinition(HBolt);
CREATE_FAST_FACTORY_STATIC(HBolt);

/**
 *  standard constructor
*/
HBolt::HBolt () : Projectile()
{
  this->registerObject(this, HBolt::_objectList);

  this->loadModel("models/projectiles/hbolt.obj",1.1);

  this->setMinEnergy(10);
  this->setHealthMax(0);
  this->lifeSpan = 7.0;

  this->angle = 0;
  this->rotationSpeed = 600;

  this->emitter = new DotEmitter(100, 5, M_2_PI);
  this->emitter->setParent(this);
  this->emitter->setSpread(M_PI, M_PI);
  this->emitter->setEmissionRate(300.0);
  this->emitter->setEmissionVelocity(50.0);

  this->mat = new Material("hBolt_halo");
  this->mat->setBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  //this->mat->setBlendFunc(GL_SRC_ALPHA,GL_ONE);
  this->mat->setDiffuse(1,1,1);
  this->mat->setAmbient(1,1,1);
  this->mat->setIllum(3);
  this->mat->setDiffuseMap("hbolt_halo.png");
//   this->mat->setDiffuseMap("hbolt_halo.png",1);
//   this->mat->setDiffuseMap("hbolt_halo.png",2);
//   dynamic_cast<StaticModel*>(this->getModel())->addMaterial(this->mat);
//   dynamic_cast<StaticModel*>(this->getModel())->finalize();

  this->halo = new Billboard();
  this->halo->setSize(.65, .65);
  this->halo->setTexture("hbolt_halo2.png");

}


/**
 *  standard deconstructor
*/
HBolt::~HBolt ()
{
  // delete this->emitter;

  /* this is normaly done by World.cc by deleting the ParticleEngine */
  if (HBolt::explosionParticles != NULL && HBolt::objectList().size() <= 1)
  {
    //if (ClassList::exists(HBolt::explosionParticles, CL_PARTICLE_SYSTEM))
    //  delete HBolt::explosionParticles;
    PRINTF(1)("Deleting HBolt Particles\n");
    HBolt::explosionParticles = NULL;
  }

}

SpriteParticles* HBolt::explosionParticles = NULL;

void HBolt::activate()
{
  if (unlikely(HBolt::explosionParticles == NULL))
  {
    HBolt::explosionParticles = new SpriteParticles(1000);
    HBolt::explosionParticles->setName("HBoltExplosionParticles");
    HBolt::explosionParticles->setLifeSpan(.5, .3);
    HBolt::explosionParticles->setRadius(0.0, 10.0);
    HBolt::explosionParticles->setRadius(.5, 6.0);
    HBolt::explosionParticles->setRadius(1.0, 3.0);
    HBolt::explosionParticles->setColor(0.0, 1,1,0,.9);
    HBolt::explosionParticles->setColor(0.5, .8,.8,0,.5);
    HBolt::explosionParticles->setColor(1.0, .8,.8,.7,.0);
  }

  this->setDamage(100);
  this->setHealth(0);
}


void HBolt::deactivate()
{
  assert (HBolt::explosionParticles != NULL);
  HBolt::explosionParticles->removeEmitter(this->emitter);
  this->lifeCycle = 0.0;

  this->toList(OM_NULL);
  this->removeNode();
  HBolt::fastFactory->kill(this);
}


void HBolt::collidesWith(WorldEntity* entity, const Vector& location)
{
  if (this->hitEntity != entity && entity->isA(CL_NPC))
    this->destroy( entity );
  this->hitEntity = entity;
}


void HBolt::updateAngle (float time)
{
  this->angle += this->rotationSpeed * time;
}


/**
 *  signal tick, time dependent things will be handled here
 * @param dt time since last tick
*/
void HBolt::tick (float dt)
{
  //Vector v = *this->flightDirection * ( this->speed * time * 1000 + 0.1);
  Vector v = this->velocity * dt;
  this->shiftCoor(v);

  if (this->tickLifeCycle(dt))
    this->deactivate();

  //float w = this->rotationSpeed * M_PI;
  //Quaternion rotation(this->rotationSpeed * dt, this->axis);
  //Quaternion u = this->getRelDir();
  //this->setRelDir(u * rotation);

  this->updateAngle(dt);

}

/**
 *  the function gets called, when the projectile is destroyed
*/
void HBolt::destroy (WorldEntity* killer)
{
  Projectile::destroy( killer );
  PRINTF(5)("DESTROY HBolt\n");
  this->lifeCycle = .95; //!< @todo calculate this usefully.

  this->emitter->setSystem(HBolt::explosionParticles);
}


void HBolt::draw () const
{

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();


  glTranslatef (this->getAbsCoor ().x,
                  this->getAbsCoor ().y,
                  this->getAbsCoor ().z);

//   glPushMatrix();
//     glEnable(GL_BLEND);
//     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//     glShadeModel(GL_FLAT);
//     glClearColor(0.0, 0.0, 0.0, 0.0);
// 
//     glEnable(GL_TEXTURE_2D);
//     this->mat->select();
// 
// //     Vector camera = State::getCamera()->getViewVector();
// 
//     glBegin(GL_QUADS); // +X
//       glTexCoord2f (0.0, 0.0);
//       glVertex3f( 0.0f, -.7f, -.7f);
// 
//       glTexCoord2f (0.0, 1.0);
//       glVertex3f( 0.0f, -.7f , .7f);
// 
//       glTexCoord2f (1.0, 1.0);
//       glVertex3f( 0.0, .7f , .7f );
// 
//       glTexCoord2f (1.0, 0.0);
//       glVertex3f( 0.0, .7f, -.7f );
//     glEnd();
//   glPopMatrix();

//   this->billboard->toggleBillboard();

  this->halo->draw();

  Vector tmpRot = this->getAbsDir().getSpacialAxis();
  glRotatef (this->getAbsDir().getSpacialAxisAngle(), tmpRot.x, tmpRot.y, tmpRot.z );
  glRotatef(this->angle, 0.0, 0.0, 1.0);

//   this->getModel()->draw();

//   this->mat->select();
   dynamic_cast<StaticModel*>(this->getModel())->draw();
//   this->mat->select();
//   dynamic_cast<StaticModel*>(this->getModel())->draw();
//   this->mat->unselect();


  glPopMatrix();
}
